import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.List;
import java.util.ArrayList;
import java.lang.StringBuilder;

public class PhoneMap
{
    /* The map of accounts to words */
    private Map<String, TreeSet<String>> acctMap;
    /* Set of winners */
    private TreeSet<String> winnerCircle;
    /* Number of accounts tied for first place */
    private int highCount;
    
    /** Constructs a hone map with an empty map, and no winners */
    public PhoneMap()
    {
        acctMap = new HashMap<String, TreeSet<String>>();
        winnerCircle = new TreeSet<String>();
        highCount = 0;
    }
   /**  Translate a word into its "account number", digits from the phone pad.
    *   The translate table is: abcdefghijklmnopqrstuvwxyz
    *                       =>  22233344455566677778889999
    * @param word String containing letters to be translated
    * @pre letters in word are lower case letters a-z
    * @return String containing the translated digits, one per letter.
    * Example:  translate("bad") returns "223"
    */
    public String translate(String word)
    {
        String translatetable = "22233344455566677778889999";
        // initialize result to empty string
        StringBuilder result = new StringBuilder();
        // For each letter in word LOOP
        for (Character ltr : word.toCharArray())
        {
            //    convert letter to an index into translate array
            int index = (int)ltr - (int)'a';
            //    lookup the digit using letter as index
            char digit = translatetable.charAt(index);
            //    append digit to result
            result.append(digit);
        }
        // END LOOP
        return result.toString();
    }

   /** Add an account number and word to the map.
    *  @param acctNum translated word (account number), string of digits 2-9.
    *  @param word original word from which acctNum was obtained.
    *  @pre translate(word).equals(acctNum)
    *  @post map is updated.  if acctNum didn't exist in map, it's added.
    *        word is added to the set of words in the value for the acctNum key.
    *  @return number of words in the set associated with acctNum
    */
    public int addNumber(String acctNum, String word)   
    {
        int wordcount = 0;
        TreeSet<String> words;
      
        // IF acctNum is in map THEN
        if (acctMap.containsKey(acctNum))
        {
            // get value for acctNum
            words = acctMap.get(acctNum);
        // ELSE // doesn't exist
        }
        else
        {
            // create new map entry with acctNum as key
            words = new TreeSet<String>();
        }
        // END IF
        // add word to list
        words.add(word);
        // put list back in the map for acctNum
        acctMap.put(acctNum, words);
        // return number of words in list (wordcount)
       return words.size();
    }
    
    /** Update the high count, keeping track of the acct num with
     * the highest number of words.
     * @param acctNum  current acct number
     * @param count    # words for this acct number
     * @post update high count, winner's circle
     */
    public void updateHighCount(String acctNum, int count)
    {
        // IF count > highCount THEN
        if (count > highCount)
        {
            highCount = count;
            // clear winner circle
            winnerCircle.clear();
            // add acctNum to winner circle
            winnerCircle.add(acctNum);
        }
        // ELSE if count equals highCount THEN
        else if (count == highCount)
        {
            //     add acctNum to winner circle
            winnerCircle.add(acctNum);
        }
        // END IF
    }
    
   /** Accessor to the list of winners.  
    *  @return list of account numbers with the most matching words
    *  in alphabetical order
    */
    public List<String> getResults()
    {
       return new ArrayList<String>(winnerCircle);
    }
   
   /** Access the list of words for a given acct number.
    *  @param acctNum translated word (account number), string of digits 2-9.
    *  @return list of words associated with acctNum in alphabetical order
    */
    public List<String> getWordsForAcct(String acctNum)
    {
        // Lookup acctNum in map, get the TreeSet of words
        return new ArrayList<String>(acctMap.get(acctNum));
    }
    /** Return a printable representation of this map for testing purposes.
     * @return String representation of this map 
     */
    public String toString()
    {
        return acctMap.toString();
    }

}
